# -*- coding: utf-8 -*-
import datetime

from pyzure.connection.connection import connect
from pyzure.create import existing_test
from pyzure import create
from pyzure.execute import execute_query
import concurrent.futures

from pyzure.send.common import cleaning_function, commit_function, get_table_info, create_table_from_info
from pyzure.tools.print_colors import C


def send_to_azure_from_one_thread(x):
    d = datetime.datetime.now()
    send_to_azure(x["instance"], x["data"], x["thread_number"], x["sub_commit"], x["table_info"])
    return (datetime.datetime.now() - d).seconds


def create_a_batch(rows, batch_size, i):
    return rows[batch_size * i:batch_size * (i + 1)]


def send_to_azure_multi_threads(instance, data, nb_threads=4, replace=True, types=None, primary_key=(),
                                sub_commit=True):
    """
    data = {
        "table_name" 	: 'name_of_the_azure_schema' + '.' + 'name_of_the_azure_table' #Must already exist,
        "columns_name" 	: [first_column_name,second_column_name,...,last_column_name],
        "rows"		: [[first_raw_value,second_raw_value,...,last_raw_value],...]
    }
    """

    # Time initialization
    start = datetime.datetime.now()

    # Extract info
    table_name = data["table_name"]
    columns_name = data["columns_name"]
    rows = data["rows"]
    total_len_data = len(rows)

    # Create table if needed
    if not existing_test(instance, table_name) or (types is not None) or (primary_key != ()):
        create.create_table(instance, data, primary_key, types)

    # Clean table if needed
    if replace:
        cleaning_function(instance, table_name)

    # Define batch size
    batch_size = int(total_len_data / nb_threads) + 1
    if total_len_data < nb_threads:
        batch_size = 1

    # Get table info
    table_info = get_table_info(instance, table_name)

    # Split data in batches of batch_size length
    split_data = []
    for i in range(nb_threads):
        split_data.append(
            {
                "data":
                    {
                        "table_name": table_name,
                        "columns_name": columns_name,
                        "rows": create_a_batch(rows, batch_size, i)
                    },
                "instance": instance,
                "thread_number": i,
                "sub_commit": sub_commit,
                "table_info": table_info,
            }
        )

    with concurrent.futures.ProcessPoolExecutor() as executor:
        r = list(executor.map(send_to_azure_from_one_thread, split_data))

    for num_thread in range(nb_threads):
        insert_query = "INSERT INTO %s SELECT * FROM %s" % (table_name, table_name + "_" + str(num_thread))
        print(insert_query)
        execute_query(instance, insert_query)

    for num_thread in range(nb_threads):
        sub_table = table_name + "_" + str(num_thread)
        print(C.HEADER + "DROP TABLE %s..." % sub_table + C.ENDC)
        execute_query(instance, "DROP TABLE %s" % sub_table)
        print(C.HEADER + "DROP TABLE %s...OK" % sub_table + C.ENDC)

    total_length_data = 0
    for element in split_data:
        total_length_data = total_length_data + len(element["data"]["rows"])

    for i in range(len(r)):
        print("Thread %s : %s seconds" % (str(i), str(r[i])))

    print("Total rows: %s" % str(total_length_data))
    print(C.BOLD + "Total time in seconds : %s" % str((datetime.datetime.now() - start).seconds) + C.ENDC)
    return 0


def send_to_azure(instance, data, thread_number, sub_commit, table_info):
    """
    data = {
        "table_name" 	: 'name_of_the_azure_schema' + '.' + 'name_of_the_azure_table' #Must already exist,
        "columns_name" 	: [first_column_name,second_column_name,...,last_column_name],
        "rows"		: [[first_raw_value,second_raw_value,...,last_raw_value],...]
    }
    """

    rows = data["rows"]
    if not rows:
        return 0
    columns_name = data["columns_name"]
    table_name = data["table_name"] + "_" + str(thread_number)

    print(C.HEADER + "Create table %s..." % table_name + C.ENDC)
    create_table_from_info(instance, table_info, table_name)
    print(C.OKGREEN + "Create table %s...OK" % table_name + C.ENDC)
    small_batch_size = int(2099 / len(columns_name))

    cnxn = connect(instance)
    cursor = cnxn.cursor()

    # Initialize counters
    boolean = True
    total_rows = len(rows)
    question_mark_pattern = "(%s)" % ",".join(["?" for i in range(len(rows[0]))])
    counter = 0
    while boolean:
        temp_row = []
        question_mark_list = []
        for i in range(small_batch_size):
            if rows:
                temp_row.append(rows.pop())
                question_mark_list.append(question_mark_pattern)
            else:
                boolean = False
                continue
        counter = counter + len(temp_row)
        percent = round(float(counter * 100) / total_rows)
        if sub_commit:
            print("Thread %s : %s %% rows sent" % (str(thread_number), str(percent)))
        else:
            print("Thread %s : %s %% rows prepared to be sent" % (str(thread_number), str(percent)))
        data_values_str = ','.join(question_mark_list)
        columns_name_str = ", ".join(columns_name)
        inserting_request = '''INSERT INTO %s (%s) VALUES %s ;''' % (table_name, columns_name_str, data_values_str)

        final_data = [y for x in temp_row for y in x]
        if final_data:
            cursor.execute(inserting_request, final_data)

        if sub_commit:
            commit_function(cnxn)
    if not sub_commit:
        commit_function(cnxn)
    cursor.close()
    cnxn.close()
    return 0


def test_multi_threads():
    data = {
        "table_name": 'testaz.test2',
        "columns_name": ["nom", "prenom", "age", "date"],
        "rows": [["pif", "pif", 12, "2017-02-23"] for v in range(1000000)]
    }
    send_to_azure_multi_threads(
        "MH_TEST",
        data,
        replace=True,
        nb_threads=4,
        sub_commit=False
    )



import pyodbc
from pyzure.connection.connection import connect


def execute_query_from_cnxn(cnxn, query, data, cursor=None, commit=True):
    if not cursor:
        cursor = cnxn.cursor()
    cursor.execute(query, data)
    if commit:
        cnxn.commit()
        cursor.close()
        return cnxn
    else:
        return cursor


def execute_query(instance, query, data=None, commit=True, cursor=None, cnxn=None, create_schema=False):
    if not cnxn and not cursor:
        cnxn = connect(instance)
        cursor = cnxn.cursor()

    if data:
        cursor.execute(query, data)
        if commit:
            cnxn.commit()
        result = None
    elif create_schema:
        cursor.execute(query)
        cnxn.commit()
        result = None
    else:
        cursor.execute(query)
        result = []
        try:
            columns = [column[0] for column in cursor.description]
            for row in cursor.fetchall():
                dict_ = dict()
                for i in range(len(columns)):
                    dict_[columns[i]] = row[i]
                result.append(dict_)
        except pyodbc.ProgrammingError:
            pass
        except TypeError:
            cnxn.commit()
            return []
        cnxn.commit()

    if commit:
        cursor.close()
        cnxn.close()

        return result
    else:
        return result, cursor, cnxn



from pyzure.connection.azure_credentials import credential
import pyodbc


def connect(instance):
    credentials = credential(instance)
    driver = credentials.get("driver")
    server = credentials.get("server")
    port = credentials.get("port")
    database = credentials.get("database")
    username = credentials.get("username")
    password = credentials.get("password")

    cnxn = pyodbc.connect(
        'DRIVER=' + driver + ';SERVER=' + server + ';PORT=' + port + ';DATABASE=' + database + ';UID=' + username + ';PWD=' + password)

    return cnxn




import pyzure
from pyzure.connection.connection import connect
from pyzure.tools.print_colors import C


def cleaning_function(instance, table_name, dev=False):
    print("Get info on table...")
    columns = get_table_info(instance, table_name)
    print("Get info on table...OK")
    cnxn = connect(instance)
    cursor = cnxn.cursor()
    drop_request = '''DROP TABLE ''' + table_name + ''';'''
    print(C.WARNING + "Drop table " + C.ENDC)
    cursor.execute(drop_request)
    cnxn.commit()
    cursor.close()
    cnxn.close()
    print("Create table from info...")
    create_table_from_info(instance, columns, table_name)
    print("Create table from info...OK")
    print(C.OKBLUE + "Cleaning Done" + C.ENDC)
    return 0


def get_table_info(instance, table_and_schema_name):
    split = table_and_schema_name.split(".")
    if len(split) == 1:
        table_name = split[0]
        schema_name = None

    elif len(split) == 2:
        table_name = split[1]
        schema_name = split[0]
    else:
        raise Exception("Invalid table or schema name")
    query = "SELECT column_name, data_type, character_maximum_length, is_nullable FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME='%s'" % table_name
    if schema_name:
        query = query + " AND TABLE_SCHEMA='%s'" % schema_name
    print(query)
    return pyzure.execute_query(instance, query)


def create_table_from_info(instance, columns, table_name):
    columns_name_string = []
    for c in columns:
        # COLUMN NAME
        c_string = c["column_name"]

        # DATA TYPE TREATMENT
        data_type = c["data_type"]
        if data_type in ("varchar"):
            data_type = data_type + "(" + str(c["character_maximum_length"]) + ")"

        c_string = c_string + " " + data_type

        # NULLABLE ?
        if c["is_nullable"] == "NO":
            c_string = c_string + " NOT NULL"

        columns_name_string.append(c_string)

    query = "CREATE TABLE %s (%s)" % (table_name, ", ".join(columns_name_string))
    print(query)
    pyzure.execute_query(instance, query)


def commit_function(cnxn):
    cnxn.commit()
    print(C.OKGREEN + "Committed" + C.ENDC)



# -*- coding: utf-8 -*-
import datetime

from pyzure.connection.connection import connect
from pyzure.create import existing_test
from pyzure import create
from pyzure.send.common import cleaning_function, commit_function
from pyzure.tools.print_colors import C


def send_to_azure(instance, data, replace=True, types=None, primary_key=(), sub_commit=True):
    """
    data = {
        "table_name" 	: 'name_of_the_azure_schema' + '.' + 'name_of_the_azure_table' #Must already exist,
        "columns_name" 	: [first_column_name,second_column_name,...,last_column_name],
        "rows"		: [[first_raw_value,second_raw_value,...,last_raw_value],...]
    }
    """

    # Time initialization
    start = datetime.datetime.now()

    # Extract info
    rows = data["rows"]
    if not rows:
        return 0
    table_name = data["table_name"]
    columns_name = data["columns_name"]
    total_len_data = len(rows)

    # Create table if needed
    if not existing_test(instance, table_name) or (types is not None) or (primary_key != ()):
        create.create_table(instance, data, primary_key, types)

    # Clean table if needed
    if replace:
        cleaning_function(instance, table_name)

    cnxn = connect(instance)
    cursor = cnxn.cursor()

    small_batch_size = int(2099 / len(columns_name))

    print("Initiate send_to_azure...")

    # Initialize counters
    boolean = True
    question_mark_pattern = "(%s)" % ",".join(["?" for i in range(len(rows[0]))])
    counter = 0
    while boolean:
        temp_row = []
        question_mark_list = []
        for i in range(small_batch_size):
            if rows:
                temp_row.append(rows.pop())
                question_mark_list.append(question_mark_pattern)
            else:
                boolean = False
                continue
        counter = counter + len(temp_row)
        percent = round(float(counter * 100) / total_len_data)
        if sub_commit:
            print("%s %% rows sent" % str(percent))
        else:
            print("%s %% rows prepared to be sent" % str(percent))
        data_values_str = ','.join(question_mark_list)
        columns_name_str = ", ".join(columns_name)
        inserting_request = '''INSERT INTO %s (%s) VALUES %s ;''' % (table_name, columns_name_str, data_values_str)

        final_data = [y for x in temp_row for y in x]
        if final_data:
            cursor.execute(inserting_request, final_data)

        if sub_commit:
            commit_function(cnxn)
    if not sub_commit:
        commit_function(cnxn)
    cursor.close()
    cnxn.close()

    print("data sent to azure")
    print("Total rows: %s" % str(total_len_data))
    print(C.BOLD + "Total time in seconds : %s" % str((datetime.datetime.now() - start).seconds) + C.ENDC)
    return 0


def test_unique_thread():
    data = {
        "table_name": 'testaz.test2',
        "columns_name": ["nom", "prenom", "age", "date"],
        "rows": [["pif", "pif", 12, "2017-02-23"] for v in range(10)]
    }
    send_to_azure(
        "MH_TEST",
        data,
        replace=True,
        sub_commit=False
    )



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

try:  # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:  # for pip <= 9.0.3
    from pip.req import parse_requirements

reqs = parse_requirements("requirements.txt", session='hack')
reqs = [str(ir.req) for ir in reqs]

with open('README.rst') as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

setup(
    name='pyzure',
    version='0.0.15',
    description='Easily send data to Microsoft Azure SQL DB',
    long_description=readme,
    author='Dacker',
    author_email='hello@dacker.co',
    url='https://github.com/dacker-team/pyzure',
    license=license,
    keywords='send data microsoft azure sql db easy',
    packages=find_packages(exclude=('tests', 'docs')),
    python_requires='>=3',
    install_requires=reqs,
)
